@@ -53,6 +53,7 @@
 #include "sheet-object-graph.h"
 #include "graph.h"
 #include "style-border.h"
+#include "style-conditions.h"
 #include "gutils.h"
 #include "expr-name.h"
 
@@ -106,6 +107,8 @@ typedef struct {
 	GPtrArray	*shared_string_array;
 	GHashTable	*styles_hash;
 	GPtrArray	*styles_array;
+	GHashTable	*dxfs_hash;
+	GPtrArray	*dxfs_array;
 	GnmConventions	*convs;
 	GOIOContext	*io_context;
 
@@ -236,15 +239,23 @@ xlsx_write_predefined_fills (GsfXMLOut *xml)
 static gint
 xlsx_find_predefined_fill (GnmStyle const *style)
 {
+	gboolean pattern_is_std =
+		(!gnm_style_is_element_set (style, MSTYLE_COLOR_PATTERN) ||
+		 gnm_style_get_pattern_color (style)->go_color == GO_COLOR_BLACK);
+	gboolean back_is_std =
+		(!gnm_style_is_element_set (style, MSTYLE_COLOR_BACK) ||
+		 gnm_style_get_back_color (style)->go_color == GO_COLOR_WHITE);
+
 	if (gnm_style_is_element_set (style, MSTYLE_PATTERN) &&
-	    gnm_style_get_pattern (style) == 0 )
+	    gnm_style_get_pattern (style) == 0 &&
+	    pattern_is_std &&
+	    back_is_std)
 		return 0;
 	if (gnm_style_is_element_set (style, MSTYLE_PATTERN) &&
 	    gnm_style_get_pattern (style) == 5 &&
-	    (!gnm_style_is_element_set (style, MSTYLE_COLOR_PATTERN) ||
-	     gnm_style_get_pattern_color (style)->go_color == GO_COLOR_BLACK) &&
-	    (!gnm_style_is_element_set (style, MSTYLE_COLOR_BACK) ||
-	     gnm_style_get_back_color (style)->go_color == GO_COLOR_WHITE))
+	    pattern_is_std &&
+	    back_is_std
+	    )
 		return 1;
 
 	return -1;
@@ -351,6 +362,16 @@ xlsx_load_buildin_num_formats (GHashTable *hash)
 			g_hash_table_insert (hash, g_strdup (std_builtins[i]), GUINT_TO_POINTER (i));
 }
 
+static void
+xlsx_write_num_format (XLSXWriteState *state, GsfXMLOut *xml,
+		       GOFormat const *format, int id)
+{
+	gsf_xml_out_start_element (xml, "numFmt");
+	gsf_xml_out_add_cstr (xml, "formatCode", go_format_as_XL (format));
+	gsf_xml_out_add_int (xml, "numFmtId", id);
+	gsf_xml_out_end_element (xml);
+}
+
 static GHashTable *
 xlsx_write_num_formats (XLSXWriteState *state, GsfXMLOut *xml)
 {
@@ -358,7 +379,6 @@ xlsx_write_num_formats (XLSXWriteState *state, GsfXMLOut *xml)
 		(g_str_hash,  g_str_equal, g_free, NULL);
 	GHashTable *num_format_hash = g_hash_table_new
 		(g_direct_hash, g_direct_equal);
-	unsigned int n = NUM_FORMAT_BASE;
 	unsigned int i, count;
 	GPtrArray *num_format_array = g_ptr_array_new ();
 
@@ -372,16 +392,15 @@ xlsx_write_num_formats (XLSXWriteState *state, GsfXMLOut *xml)
 
 		if (!gnm_style_is_element_set (style, MSTYLE_FORMAT))
 			continue;
-		format =  gnm_style_get_format (style);
+		format = gnm_style_get_format (style);
 		if (go_format_is_general (format))
 			continue;
 		xl = go_format_as_XL (format);
 		tmp = g_hash_table_lookup (hash, xl);
 		if (tmp == NULL) {
-			char *xl_c =  g_strdup (xl);
-			tmp = GUINT_TO_POINTER (n++);
-			g_hash_table_insert (hash, xl_c, tmp);
-			g_ptr_array_add (num_format_array, xl_c);
+			tmp = GUINT_TO_POINTER (num_format_array->len + NUM_FORMAT_BASE);
+			g_hash_table_insert (hash, g_strdup (xl), tmp);
+			g_ptr_array_add (num_format_array, (gpointer)format);
 		}
 		g_hash_table_insert (num_format_hash, (gpointer) style, tmp);
 	}
@@ -390,13 +409,11 @@ xlsx_write_num_formats (XLSXWriteState *state, GsfXMLOut *xml)
 	if (count != 0) {
 		gsf_xml_out_start_element (xml, "numFmts");
 		gsf_xml_out_add_int (xml, "count", count);
-		for (i = 0 , n = NUM_FORMAT_BASE; i < count ; i++, n++) {
-			gsf_xml_out_start_element (xml, "numFmt");
-			gsf_xml_out_add_cstr
-				(xml, "formatCode",
-				 g_ptr_array_index (num_format_array, i));
-			gsf_xml_out_add_int (xml, "numFmtId", n);
-			gsf_xml_out_end_element (xml);
+		for (i = 0; i < count; i++) {
+			GOFormat const *format =
+				g_ptr_array_index (num_format_array, i);
+			xlsx_write_num_format (state, xml, format,
+					       i + NUM_FORMAT_BASE);
 		}
 		gsf_xml_out_end_element (xml);
 	}
@@ -419,58 +436,111 @@ xlsx_find_font (GnmStyle const *style, GPtrArray  *styles)
 	unsigned i;
 	for (i = 0 ; i < styles->len ; i++) {
 		GnmStyle const *old_style = g_ptr_array_index (styles, i);
-		if (style == old_style)
-			return (gint) i;
-		if (gnm_style_is_element_set (style, MSTYLE_FONT_BOLD) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_FONT_BOLD) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_FONT_ITALIC) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_FONT_UNDERLINE) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_COLOR) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_FONT_COLOR) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_NAME) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_FONT_NAME) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_SIZE) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_FONT_SIZE) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_SCRIPT) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_FONT_SCRIPT) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_FONT_STRIKETHROUGH))
-			continue;
-		if (gnm_style_get_font_bold (style) != gnm_style_get_font_bold (old_style) ||
-		    gnm_style_get_font_italic (style) != gnm_style_get_font_italic (old_style) ||
-		    gnm_style_get_font_uline (style) != gnm_style_get_font_uline (old_style) ||
-		    gnm_style_get_font_color (style)->go_color !=
-		    gnm_style_get_font_color (old_style)->go_color ||
-		    gnm_style_get_font_size (style) != gnm_style_get_font_size (old_style) ||
-		    gnm_style_get_font_script (style) != gnm_style_get_font_script (old_style) ||
-		    gnm_style_get_font_strike (style) != gnm_style_get_font_strike (old_style))
-			continue;
-		if (0 == strcmp (gnm_style_get_font_name (style),
-				 gnm_style_get_font_name (old_style)))
+		if (style == old_style ||
+		    (gnm_style_equal_elem (style, old_style, MSTYLE_FONT_BOLD) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_FONT_ITALIC) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_FONT_UNDERLINE) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_FONT_COLOR) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_FONT_SIZE) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_FONT_SCRIPT) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_FONT_STRIKETHROUGH) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_FONT_NAME)))
 			return (gint) i;
 	}
 	return -1;
 }
 
+static gboolean
+xlsx_has_font_style (GnmStyle const *style)
+{
+	return (gnm_style_is_element_set (style, MSTYLE_FONT_BOLD) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_COLOR) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_NAME) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_SCRIPT) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_SIZE) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH));
+}
+
+static void
+xlsx_write_font (XLSXWriteState *state, GsfXMLOut *xml, GnmStyle const *style)
+{
+	gsf_xml_out_start_element (xml, "font");
+	if (gnm_style_is_element_set (style, MSTYLE_FONT_BOLD)) {
+		gsf_xml_out_start_element (xml, "b");
+		xlsx_add_bool (xml, "val", gnm_style_get_font_bold (style));
+		gsf_xml_out_end_element (xml);
+	}
+	if (gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC)) {
+		gsf_xml_out_start_element (xml, "i");
+		xlsx_add_bool (xml, "val", gnm_style_get_font_italic (style));
+		gsf_xml_out_end_element (xml);
+	}
+	if (gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE)) {
+		static const char * const types[] = {
+			"none",
+			"single",
+			"double",
+			"singleAccounting",
+			"doubleAccounting"
+		};
+
+		gsf_xml_out_start_element (xml, "u");
+		gsf_xml_out_add_cstr
+			(xml, "val", types[gnm_style_get_font_uline (style)]);
+		gsf_xml_out_end_element (xml);
+	}
+	if (gnm_style_is_element_set (style, MSTYLE_FONT_COLOR))
+		xlsx_write_color_element
+			(xml, "color",
+			 gnm_style_get_font_color (style)->go_color);
+	if (gnm_style_is_element_set (style, MSTYLE_FONT_NAME)) {
+		gsf_xml_out_start_element (xml, "name");
+		gsf_xml_out_add_cstr
+			(xml, "val", gnm_style_get_font_name (style));
+		gsf_xml_out_end_element (xml);
+	}
+	if (gnm_style_is_element_set (style, MSTYLE_FONT_SCRIPT)) {
+		GOFontScript scr = gnm_style_get_font_script (style);
+		const char *val;
+
+		switch (scr) {
+		case GO_FONT_SCRIPT_SUB: val = "subscript"; break;
+		case GO_FONT_SCRIPT_SUPER: val = "superscript"; break;
+		default:
+		case GO_FONT_SCRIPT_STANDARD: val = "baseline"; break;
+		}
+		gsf_xml_out_start_element (xml, "vertAlign");
+		gsf_xml_out_add_cstr (xml, "val", val);
+		gsf_xml_out_end_element (xml);
+	}
+	if (gnm_style_is_element_set (style, MSTYLE_FONT_SIZE)) {
+		gsf_xml_out_start_element (xml, "sz");
+		gsf_xml_out_add_float
+			(xml, "val", gnm_style_get_font_size (style),
+			 2);
+		gsf_xml_out_end_element (xml);
+
+	}
+	if (gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH)) {
+		gsf_xml_out_start_element (xml, "strike");
+		xlsx_add_bool (xml, "val", gnm_style_get_font_strike (style));
+		gsf_xml_out_end_element (xml);
+	}
+	gsf_xml_out_end_element (xml); /* font */
+}
+
 static GHashTable *
 xlsx_write_fonts (XLSXWriteState *state, GsfXMLOut *xml)
 {
-	GHashTable *fonts_hash =  g_hash_table_new (g_direct_hash, g_direct_equal);
-	GPtrArray  *styles_w_fonts  = g_ptr_array_new ();
+	GHashTable *fonts_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	GPtrArray  *styles_w_fonts = g_ptr_array_new ();
 	unsigned int i;
 
 	for (i = 0 ; i < state->styles_array->len ; i++) {
 		GnmStyle const *style = g_ptr_array_index (state->styles_array, i);
-		if (gnm_style_is_element_set (style, MSTYLE_FONT_BOLD) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_COLOR) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_NAME) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_SCRIPT) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_SIZE) ||
-		    gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH)) {
+		if (xlsx_has_font_style (style)) {
 			gint font_n = xlsx_find_font (style, styles_w_fonts);
 			if (font_n < 0) {
 				g_ptr_array_add (styles_w_fonts, (gpointer)style);
@@ -487,70 +557,7 @@ xlsx_write_fonts (XLSXWriteState *state, GsfXMLOut *xml)
 		gsf_xml_out_add_int (xml, "count", styles_w_fonts->len);
 		for (i = 0 ; i < styles_w_fonts->len ; i++) {
 			GnmStyle const *style = g_ptr_array_index (styles_w_fonts, i);
-			gsf_xml_out_start_element (xml, "font");
-			if (gnm_style_is_element_set (style, MSTYLE_FONT_BOLD)) {
-				gsf_xml_out_start_element (xml, "b");
-				xlsx_add_bool (xml, "val", gnm_style_get_font_bold (style));
-				gsf_xml_out_end_element (xml);
-			}
-			if (gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC)) {
-				gsf_xml_out_start_element (xml, "i");
-				xlsx_add_bool (xml, "val", gnm_style_get_font_italic (style));
-				gsf_xml_out_end_element (xml);
-			}
-			if (gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE)) {
-				static const char * const types[] = {
-					"none",
-					"single",
-					"double",
-					"singleAccounting",
-					"doubleAccounting"
-				};
-
-				gsf_xml_out_start_element (xml, "u");
-				gsf_xml_out_add_cstr
-					(xml, "val", types[gnm_style_get_font_uline (style)]);
-				gsf_xml_out_end_element (xml);
-			}
-			if (gnm_style_is_element_set (style, MSTYLE_FONT_COLOR))
-				xlsx_write_color_element
-					(xml, "color",
-					 gnm_style_get_font_color (style)->go_color);
-			if (gnm_style_is_element_set (style, MSTYLE_FONT_NAME)) {
-				gsf_xml_out_start_element (xml, "name");
-				gsf_xml_out_add_cstr
-					(xml, "val", gnm_style_get_font_name (style));
-				gsf_xml_out_end_element (xml);
-			}
-			if (gnm_style_is_element_set (style, MSTYLE_FONT_SCRIPT)) {
-				GOFontScript scr = gnm_style_get_font_script (style);
-				const char *val;
-
-				switch (scr) {
-				case GO_FONT_SCRIPT_SUB: val = "subscript"; break;
-				case GO_FONT_SCRIPT_SUPER: val = "superscript"; break;
-				default: val = NULL;
-				}
-				if (val) {
-					gsf_xml_out_start_element (xml, "vertAlign");
-					gsf_xml_out_add_cstr (xml, "val", val);
-					gsf_xml_out_end_element (xml);
-				}
-			}
-			if (gnm_style_is_element_set (style, MSTYLE_FONT_SIZE)) {
-				gsf_xml_out_start_element (xml, "sz");
-				gsf_xml_out_add_float
-					(xml, "val", gnm_style_get_font_size (style),
-					 2);
-				gsf_xml_out_end_element (xml);
-
-			}
-			if (gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH)) {
-				gsf_xml_out_start_element (xml, "strike");
-				xlsx_add_bool (xml, "val", gnm_style_get_font_strike (style));
-				gsf_xml_out_end_element (xml);
-			}
-			gsf_xml_out_end_element (xml); /* font */
+			xlsx_write_font (state, xml, style);
 		}
 		gsf_xml_out_end_element (xml);
 	}
@@ -570,27 +577,71 @@ xlsx_find_fill (GnmStyle const *style, GPtrArray *styles)
 		return j;
 	for (i = 0 ; i < styles->len ; i++) {
 		GnmStyle const *old_style = g_ptr_array_index (styles, i);
-		if (style == old_style)
-			return (gint) i;
-		if (!(gnm_style_is_element_set (style, MSTYLE_COLOR_BACK) !=
-		      gnm_style_is_element_set (old_style, MSTYLE_COLOR_BACK)
-		      || gnm_style_is_element_set (style, MSTYLE_COLOR_PATTERN) !=
-		      gnm_style_is_element_set (old_style, MSTYLE_COLOR_PATTERN)
-		      || gnm_style_is_element_set (style, MSTYLE_PATTERN) !=
-		      gnm_style_is_element_set (style, MSTYLE_PATTERN)
-		      || (gnm_style_is_element_set (style, MSTYLE_PATTERN) &&
-			  gnm_style_get_pattern (style) != gnm_style_get_pattern (old_style))
-		      || (gnm_style_is_element_set (style, MSTYLE_COLOR_BACK) &&
-			  gnm_style_get_back_color (style)->go_color !=
-			  gnm_style_get_back_color (old_style)->go_color)
-		      || (gnm_style_is_element_set (style, MSTYLE_COLOR_PATTERN) &&
-			  gnm_style_get_pattern_color (style)->go_color !=
-			  gnm_style_get_pattern_color (old_style)->go_color)))
+		if (style == old_style ||
+		    (gnm_style_equal_elem (style, old_style, MSTYLE_COLOR_BACK) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_COLOR_PATTERN) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_PATTERN)))
 			return (gint) i + N_PREDEFINED_FILLS;
 	}
 	return -1;
 }
 
+static gboolean
+xlsx_has_background_style (GnmStyle const *style)
+{
+	return (gnm_style_is_element_set (style, MSTYLE_COLOR_BACK) ||
+		gnm_style_is_element_set (style, MSTYLE_COLOR_PATTERN) ||
+		gnm_style_is_element_set (style, MSTYLE_PATTERN));
+}
+
+static void
+xlsx_write_background (XLSXWriteState *state, GsfXMLOut *xml,
+		       GnmStyle const *style, gboolean invert_solid)
+{
+	/*
+	 * MAGIC:
+	 * Looks like pattern background and forground colours are inverted
+	 * for dxfs with solid fills for no apparent reason.
+	 */
+	gboolean invert = FALSE;
+	GnmColor *fg;
+	GnmColor *bg;
+
+	gsf_xml_out_start_element (xml, "fill");
+	gsf_xml_out_start_element (xml, "patternFill");
+	if (gnm_style_is_element_set (style, MSTYLE_PATTERN)) {
+		gint pattern = gnm_style_get_pattern (style);
+		const char *type;
+		if (pattern <= 0 || pattern > (gint)G_N_ELEMENTS (pats)) {
+			type = "none";
+		} else {
+			invert = (pattern == 1 && invert_solid);
+			type = pats[pattern - 1];
+		}
+		gsf_xml_out_add_cstr_unchecked (xml, "patternType", type);
+	}
+
+	fg = gnm_style_is_element_set (style, MSTYLE_COLOR_BACK)
+		? gnm_style_get_back_color (style)
+		: NULL;
+	bg = gnm_style_is_element_set (style, MSTYLE_COLOR_PATTERN)
+		? gnm_style_get_pattern_color (style)
+		: NULL;
+	if (invert) {
+		GnmColor *tmp = fg;
+		fg = bg;
+		bg = tmp;
+	}
+
+	if (fg)
+		xlsx_write_color_element (xml, "fgColor", fg->go_color);
+	if (bg)
+		xlsx_write_color_element (xml, "bgColor", bg->go_color);
+
+	gsf_xml_out_end_element (xml);
+	gsf_xml_out_end_element (xml);
+}
+
 static GHashTable *
 xlsx_write_fills (XLSXWriteState *state, GsfXMLOut *xml)
 {
@@ -600,9 +651,7 @@ xlsx_write_fills (XLSXWriteState *state, GsfXMLOut *xml)
 
 	for (i = 0 ; i < state->styles_array->len ; i++) {
 		GnmStyle const *style = g_ptr_array_index (state->styles_array, i);
-		if (gnm_style_is_element_set (style, MSTYLE_COLOR_BACK) ||
-		    gnm_style_is_element_set (style, MSTYLE_COLOR_PATTERN) ||
-		    gnm_style_is_element_set (style, MSTYLE_PATTERN)) {
+		if (xlsx_has_background_style (style)) {
 			gint fill_n = xlsx_find_fill (style, styles_w_fills);
 			if (fill_n < 0) {
 				g_ptr_array_add (styles_w_fills, (gpointer)style);
@@ -626,46 +675,7 @@ xlsx_write_fills (XLSXWriteState *state, GsfXMLOut *xml)
 	xlsx_write_predefined_fills (xml);
 	for (i = 0 ; i < styles_w_fills->len ; i++) {
 		GnmStyle const *style = g_ptr_array_index (styles_w_fills, i);
-		gsf_xml_out_start_element (xml, "fill");
-		gsf_xml_out_start_element (xml, "patternFill");
-		if (gnm_style_is_element_set (style, MSTYLE_PATTERN)) {
-			gint pattern = gnm_style_get_pattern (style);
-			switch (pattern) {
-			case 0:
-				gsf_xml_out_add_cstr_unchecked (xml, "patternType",
-								"none");
-				break;
-			case 1:
-				gsf_xml_out_add_cstr_unchecked (xml, "patternType",
-								"solid");
-				if (gnm_style_is_element_set (style, MSTYLE_COLOR_BACK)) {
-					xlsx_write_color_element
-						(xml, "fgColor",
-						 gnm_style_get_back_color (style)->go_color);
-				}
-				break;
-			default:
-				if (pattern < 2 || pattern > (gint)G_N_ELEMENTS (pats))
-					gsf_xml_out_add_cstr_unchecked (xml, "patternType",
-									"none");
-				else
-					gsf_xml_out_add_cstr_unchecked (xml, "patternType",
-									pats[pattern - 1]);
-				break;
-			}
-			if (pattern > 1) {
-				if (gnm_style_is_element_set (style, MSTYLE_COLOR_BACK))
-					xlsx_write_color_element
-						(xml, "fgColor",
-						 gnm_style_get_back_color (style)->go_color);
-				if (gnm_style_is_element_set (style, MSTYLE_COLOR_PATTERN))
-					xlsx_write_color_element
-						(xml, "bgColor",
-						 gnm_style_get_pattern_color (style)->go_color);
-			}
-		}
-		gsf_xml_out_end_element (xml);
-		gsf_xml_out_end_element (xml);
+		xlsx_write_background (state, xml, style, FALSE);
 	}
 	gsf_xml_out_end_element (xml);
 
@@ -679,35 +689,14 @@ xlsx_find_border (GnmStyle const *style, GPtrArray  *styles)
 	unsigned i;
 	for (i = 0 ; i < styles->len ; i++) {
 		GnmStyle const *old_style = g_ptr_array_index (styles, i);
-		if (style == old_style)
+		if (style == old_style ||
+		    (gnm_style_equal_elem (style, old_style, MSTYLE_BORDER_TOP) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_BORDER_BOTTOM) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_BORDER_LEFT) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_BORDER_RIGHT) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_BORDER_DIAGONAL) &&
+		     gnm_style_equal_elem (style, old_style, MSTYLE_BORDER_REV_DIAGONAL)))
 			return (gint) i;
-		if (gnm_style_is_element_set (style, MSTYLE_BORDER_TOP) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_BORDER_TOP) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_BOTTOM) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_BORDER_BOTTOM) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_LEFT) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_BORDER_LEFT) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_RIGHT) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_BORDER_RIGHT) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_REV_DIAGONAL) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_BORDER_REV_DIAGONAL) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_DIAGONAL) !=
-		    gnm_style_is_element_set (old_style, MSTYLE_BORDER_DIAGONAL))
-			continue;
-		if (gnm_style_get_border (style, MSTYLE_BORDER_TOP)
-		    != gnm_style_get_border (old_style, MSTYLE_BORDER_TOP) ||
-		    gnm_style_get_border (style, MSTYLE_BORDER_BOTTOM)
-		    != gnm_style_get_border (old_style, MSTYLE_BORDER_BOTTOM) ||
-		    gnm_style_get_border (style, MSTYLE_BORDER_LEFT)
-		    != gnm_style_get_border (old_style, MSTYLE_BORDER_LEFT) ||
-		    gnm_style_get_border (style, MSTYLE_BORDER_RIGHT)
-		    != gnm_style_get_border (old_style, MSTYLE_BORDER_RIGHT) ||
-		    gnm_style_get_border (style, MSTYLE_BORDER_REV_DIAGONAL)
-		    != gnm_style_get_border (old_style, MSTYLE_BORDER_REV_DIAGONAL) ||
-		    gnm_style_get_border (style, MSTYLE_BORDER_DIAGONAL)
-		    != gnm_style_get_border (old_style, MSTYLE_BORDER_DIAGONAL))
-			continue;
-		return (gint) i;
 	}
 	return -1;
 }
@@ -790,6 +779,81 @@ xlsx_write_border (XLSXWriteState *state, GsfXMLOut *xml, GnmBorder *border, Gnm
 	gsf_xml_out_end_element (xml);
 }
 
+static gboolean
+xlsx_has_border_style (GnmStyle const *style)
+{
+	return (gnm_style_is_element_set (style, MSTYLE_BORDER_TOP) ||
+		gnm_style_is_element_set (style, MSTYLE_BORDER_BOTTOM) ||
+		gnm_style_is_element_set (style, MSTYLE_BORDER_LEFT) ||
+		gnm_style_is_element_set (style, MSTYLE_BORDER_RIGHT) ||
+		gnm_style_is_element_set (style, MSTYLE_BORDER_REV_DIAGONAL) ||
+		gnm_style_is_element_set (style, MSTYLE_BORDER_DIAGONAL));
+}
+
+static void
+xlsx_write_full_border (XLSXWriteState *state, GsfXMLOut *xml,
+			GnmStyle const *style)
+{
+	gboolean diagonal_border_written = FALSE;
+	gsf_xml_out_start_element (xml, "border");
+	if (gnm_style_is_element_set (style, MSTYLE_BORDER_DIAGONAL)) {
+		GnmBorder *border = gnm_style_get_border
+			(style, MSTYLE_BORDER_DIAGONAL);
+		gsf_xml_out_add_bool
+			(xml, "diagonalUp",
+			 (border && border->line_type != GNM_STYLE_BORDER_NONE));
+	}
+	if (gnm_style_is_element_set (style, MSTYLE_BORDER_REV_DIAGONAL)) {
+		GnmBorder *border = gnm_style_get_border
+			(style, MSTYLE_BORDER_REV_DIAGONAL);
+		gsf_xml_out_add_bool
+			(xml, "diagonalDown",
+			 (border && border->line_type != GNM_STYLE_BORDER_NONE));
+	}
+	if (gnm_style_is_element_set (style, MSTYLE_BORDER_LEFT))
+		xlsx_write_border (state, xml,
+				   gnm_style_get_border
+				   (style, MSTYLE_BORDER_LEFT),
+				   MSTYLE_BORDER_LEFT);
+	if (gnm_style_is_element_set (style, MSTYLE_BORDER_RIGHT))
+		xlsx_write_border (state, xml,
+				   gnm_style_get_border
+				   (style, MSTYLE_BORDER_RIGHT),
+				   MSTYLE_BORDER_RIGHT);
+	if (gnm_style_is_element_set (style, MSTYLE_BORDER_TOP))
+		xlsx_write_border (state, xml,
+				   gnm_style_get_border
+				   (style, MSTYLE_BORDER_TOP),
+				   MSTYLE_BORDER_TOP);
+	if (gnm_style_is_element_set (style, MSTYLE_BORDER_BOTTOM))
+		xlsx_write_border (state, xml,
+				   gnm_style_get_border
+				   (style, MSTYLE_BORDER_BOTTOM),
+				   MSTYLE_BORDER_BOTTOM);
+	if (gnm_style_is_element_set (style, MSTYLE_BORDER_DIAGONAL)) {
+		GnmBorder *border = gnm_style_get_border
+			(style, MSTYLE_BORDER_DIAGONAL);
+		if (border && border->line_type != GNM_STYLE_BORDER_NONE) {
+			diagonal_border_written = TRUE;
+			xlsx_write_border (state, xml,
+					   border,
+					   MSTYLE_BORDER_DIAGONAL);
+		}
+	}
+	if (!diagonal_border_written &&
+	    gnm_style_is_element_set (style, MSTYLE_BORDER_REV_DIAGONAL)) {
+		GnmBorder *border = gnm_style_get_border
+			(style, MSTYLE_BORDER_REV_DIAGONAL);
+		if (border && border->line_type != GNM_STYLE_BORDER_NONE) {
+			xlsx_write_border (state, xml,
+					   border,
+					   MSTYLE_BORDER_REV_DIAGONAL);
+		}
+	}
+	gsf_xml_out_end_element (xml); /* border */
+}
+
+
 static GHashTable *
 xlsx_write_borders (XLSXWriteState *state, GsfXMLOut *xml)
 {
@@ -799,12 +863,7 @@ xlsx_write_borders (XLSXWriteState *state, GsfXMLOut *xml)
 
 	for (i = 0 ; i < state->styles_array->len ; i++) {
 		GnmStyle const *style = g_ptr_array_index (state->styles_array, i);
-		if (gnm_style_is_element_set (style, MSTYLE_BORDER_TOP) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_BOTTOM) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_LEFT) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_RIGHT) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_REV_DIAGONAL) ||
-		    gnm_style_is_element_set (style, MSTYLE_BORDER_DIAGONAL)) {
+		if (xlsx_has_border_style (style)) {
 			gint border_n = xlsx_find_border (style, styles_w_border);
 			if (border_n < 0) {
 				g_ptr_array_add (styles_w_border, (gpointer)style);
@@ -819,65 +878,9 @@ xlsx_write_borders (XLSXWriteState *state, GsfXMLOut *xml)
 	if (styles_w_border->len > 0) {
 		gsf_xml_out_start_element (xml, "borders");
 		gsf_xml_out_add_int (xml, "count", styles_w_border->len);
-		for (i = 0 ; i < styles_w_border->len ; i++) {
+		for (i = 0; i < styles_w_border->len; i++) {
 			GnmStyle const *style = g_ptr_array_index (styles_w_border, i);
-			gboolean diagonal_border_written = FALSE;
-			gsf_xml_out_start_element (xml, "border");
-			if (gnm_style_is_element_set (style, MSTYLE_BORDER_DIAGONAL)) {
-				GnmBorder *border = gnm_style_get_border
-					(style, MSTYLE_BORDER_DIAGONAL);
-				gsf_xml_out_add_bool
-					(xml, "diagonalUp",
-					 (border && border->line_type != GNM_STYLE_BORDER_NONE));
-			}
-			if (gnm_style_is_element_set (style, MSTYLE_BORDER_REV_DIAGONAL)) {
-				GnmBorder *border = gnm_style_get_border
-					(style, MSTYLE_BORDER_REV_DIAGONAL);
-				gsf_xml_out_add_bool
-					(xml, "diagonalDown",
-					 (border && border->line_type != GNM_STYLE_BORDER_NONE));
-			}
-			if (gnm_style_is_element_set (style, MSTYLE_BORDER_LEFT))
-				xlsx_write_border (state, xml,
-						   gnm_style_get_border
-						   (style, MSTYLE_BORDER_LEFT),
-						   MSTYLE_BORDER_LEFT);
-			if (gnm_style_is_element_set (style, MSTYLE_BORDER_RIGHT))
-				xlsx_write_border (state, xml,
-						   gnm_style_get_border
-						   (style, MSTYLE_BORDER_RIGHT),
-						   MSTYLE_BORDER_RIGHT);
-			if (gnm_style_is_element_set (style, MSTYLE_BORDER_TOP))
-				xlsx_write_border (state, xml,
-						   gnm_style_get_border
-						   (style, MSTYLE_BORDER_TOP),
-						   MSTYLE_BORDER_TOP);
-			if (gnm_style_is_element_set (style, MSTYLE_BORDER_BOTTOM))
-				xlsx_write_border (state, xml,
-						   gnm_style_get_border
-						   (style, MSTYLE_BORDER_BOTTOM),
-						   MSTYLE_BORDER_BOTTOM);
-			if (gnm_style_is_element_set (style, MSTYLE_BORDER_DIAGONAL)) {
-				GnmBorder *border = gnm_style_get_border
-					(style, MSTYLE_BORDER_DIAGONAL);
-				if (border && border->line_type != GNM_STYLE_BORDER_NONE) {
-					diagonal_border_written = TRUE;
-					xlsx_write_border (state, xml,
-						   border,
-						   MSTYLE_BORDER_DIAGONAL);
-				}
-			}
-			if (!diagonal_border_written &&
-			    gnm_style_is_element_set (style, MSTYLE_BORDER_REV_DIAGONAL)) {
-				GnmBorder *border = gnm_style_get_border
-					(style, MSTYLE_BORDER_REV_DIAGONAL);
-				if (border && border->line_type != GNM_STYLE_BORDER_NONE) {
-					xlsx_write_border (state, xml,
-						   border,
-						   MSTYLE_BORDER_REV_DIAGONAL);
-				}
-			}
-			gsf_xml_out_end_element (xml); /* border */
+			xlsx_write_full_border (state, xml, style);
 		}
 		gsf_xml_out_end_element (xml);
 	}
@@ -887,7 +890,7 @@ xlsx_write_borders (XLSXWriteState *state, GsfXMLOut *xml)
 }
 
 static gboolean
-xlsx_write_style_want_alignment (GnmStyle const *style)
+xlsx_has_alignment_style (GnmStyle const *style)
 {
 	return gnm_style_is_element_set (style, MSTYLE_ALIGN_H)
 		|| gnm_style_is_element_set (style, MSTYLE_ALIGN_V)
@@ -990,7 +993,7 @@ xlsx_write_style (XLSXWriteState *state, GsfXMLOut *xml,
 		  GHashTable *num_format_hash, GHashTable *fonts_hash,
 		  GHashTable *border_hash, gint id)
 {
-	gboolean alignment = xlsx_write_style_want_alignment (style);
+	gboolean alignment = xlsx_has_alignment_style (style);
 	gpointer tmp_fill, tmp_font, tmp_border;
 	gboolean fill = (NULL != (tmp_fill = g_hash_table_lookup (fills_hash, style)));
 	gboolean font = (NULL != (tmp_font = g_hash_table_lookup (fonts_hash, style)));
@@ -1019,7 +1022,6 @@ xlsx_write_style (XLSXWriteState *state, GsfXMLOut *xml,
 
 	if (alignment)
 		xlsx_write_style_write_alignment (state, xml, style);
-
 }
 
 static void
@@ -1044,21 +1046,58 @@ xlsx_write_cellXfs (XLSXWriteState *state, GsfXMLOut *xml,
 		    GHashTable *fills_hash, GHashTable *num_format_hash,
 		    GHashTable *fonts_hash, GHashTable *border_hash)
 {
-	if (state->styles_array->len > 0) {
-		unsigned i;
-		gsf_xml_out_start_element (xml, "cellXfs");
-		gsf_xml_out_add_int (xml, "count", state->styles_array->len);
-		for (i = 0 ; i < state->styles_array->len ; i++) {
-			gsf_xml_out_start_element (xml, "xf");
-			xlsx_write_style
-				(state, xml,
-				 g_ptr_array_index (state->styles_array, i),
-				 fills_hash, num_format_hash, fonts_hash,
-				 border_hash, 0);
-			gsf_xml_out_end_element (xml);
-		}
+	unsigned i, N = state->styles_array->len;
+
+	if (N == 0)
+		return;
+
+	gsf_xml_out_start_element (xml, "cellXfs");
+	gsf_xml_out_add_int (xml, "count", N);
+	for (i = 0; i < N; i++) {
+		GnmStyle const *style =
+			g_ptr_array_index (state->styles_array, i);
+		gsf_xml_out_start_element (xml, "xf");
+		xlsx_write_style
+			(state, xml, style,
+			 fills_hash, num_format_hash, fonts_hash,
+			 border_hash, 0);
 		gsf_xml_out_end_element (xml);
 	}
+	gsf_xml_out_end_element (xml);
+}
+
+static void
+xlsx_write_dxfs (XLSXWriteState *state, GsfXMLOut *xml,
+		 GHashTable *num_format_hash)
+{
+	unsigned i, N = state->dxfs_array->len;
+	int id = NUM_FORMAT_BASE + g_hash_table_size (num_format_hash);
+
+	if (N == 0)
+		return;
+
+	gsf_xml_out_start_element (xml, "dxfs");
+	gsf_xml_out_add_int (xml, "count", N);
+	for (i = 0; i < N; i++) {
+		GnmStyle *style =
+			g_ptr_array_index (state->dxfs_array, i);
+		gsf_xml_out_start_element (xml, "dxf");
+		if (xlsx_has_font_style (style))
+			xlsx_write_font (state, xml, style);
+		if (xlsx_has_alignment_style (style))
+			xlsx_write_style_write_alignment (state, xml, style);
+		if (xlsx_has_background_style (style))
+			xlsx_write_background (state, xml, style, TRUE);
+		if (gnm_style_is_element_set (style, MSTYLE_FORMAT)) {
+			GOFormat const *format = gnm_style_get_format (style);
+			xlsx_write_num_format (state, xml, format, id++);
+		}
+		if (xlsx_has_border_style (style))
+			xlsx_write_full_border (state, xml, style);
+
+		gsf_xml_out_end_element (xml);
+	}
+	gsf_xml_out_end_element (xml);
 }
 
 static void
@@ -1083,7 +1122,7 @@ xlsx_write_styles (XLSXWriteState *state, GsfOutfile *wb_part)
 	xlsx_write_cellStyleXfs (state, xml, fills_hash, num_format_hash, fonts_hash, border_hash);
 	xlsx_write_cellXfs (state, xml, fills_hash, num_format_hash, fonts_hash, border_hash);
 	/* <xsd:element name="cellStyles" type="CT_CellStyles" minOccurs="0" maxOccurs="1"/> */
-	/* <xsd:element name="dxfs" type="CT_Dxfs" minOccurs="0" maxOccurs="1"/> */
+	xlsx_write_dxfs (state, xml, num_format_hash);
 	/* <xsd:element name="tableStyles" type="CT_TableStyles" minOccurs="0" maxOccurs="1"/> */
 	/* <xsd:element name="colors" type="CT_Colors" minOccurs="0" maxOccurs="1"/> */
 	/* <xsd:element name="extLst" type="CT_ExtensionList" minOccurs="0" maxOccurs="1"/> */
@@ -1227,6 +1266,21 @@ xlsx_get_style_id (XLSXWriteState *state, GnmStyle const *style)
 	return GPOINTER_TO_INT (tmp) - 1;
 }
 
+static gint
+xlsx_get_cond_style_id (XLSXWriteState *state, GnmStyle const *style)
+{
+	gpointer tmp;
+
+	g_return_val_if_fail (style != NULL, 0);
+
+	if (NULL == (tmp = g_hash_table_lookup (state->dxfs_hash, style))) {
+		g_ptr_array_add (state->dxfs_array, (gpointer)style);
+		tmp = GINT_TO_POINTER (state->dxfs_array->len);
+		g_hash_table_insert (state->dxfs_hash, (gpointer)style, tmp);
+	}
+	return GPOINTER_TO_INT (tmp) - 1;
+}
+
 static gboolean
 row_boring (Sheet *sheet, int r)
 {
@@ -1350,11 +1404,19 @@ xlsx_write_cells (XLSXWriteState *state, GsfXMLOut *xml,
 
 				switch (val->type) {
 				default :
-				case VALUE_EMPTY :	type = NULL; break; /* FIXME : what to do ? */
-				case VALUE_BOOLEAN :	type = "b"; break;
-				case VALUE_FLOAT :	type = ""; break; /* "n" is the default */
-				case VALUE_ERROR :	type = "e"; break;
-				case VALUE_STRING :
+				case VALUE_EMPTY:
+					type = NULL; /* FIXME : what to do ? */
+					break;
+				case VALUE_BOOLEAN:
+					type = "b";
+					break;
+				case VALUE_FLOAT:
+					type = ""; /* "n" is the default */
+					break;
+				case VALUE_ERROR:
+					type = "e";
+					break;
+				case VALUE_STRING:
 					/* A reasonable approximation of * 'is_shared'.  It can get spoofed by
 					 * rich text references to a base * string */
 					if (go_string_get_ref_count (val->v_str.val) > 1) {
@@ -1368,8 +1430,10 @@ xlsx_write_cells (XLSXWriteState *state, GsfXMLOut *xml,
 					} else
 						type = "str";
 					break;
-				case VALUE_CELLRANGE :
-				case VALUE_ARRAY :	type = NULL; break;	/* FIXME */
+				case VALUE_CELLRANGE:
+				case VALUE_ARRAY:
+					type = NULL; /* FIXME */
+					break;
 				}
 
 				if (NULL != type && *type)
@@ -1401,13 +1465,14 @@ xlsx_write_cells (XLSXWriteState *state, GsfXMLOut *xml,
 					if (str_id >= 0) {
 						gsf_xml_out_add_int (xml, NULL, str_id);
 						str_id = -1;
-					} else if (val->type != VALUE_BOOLEAN) {
+					} else if (VALUE_IS_BOOLEAN (val)) {
+						xlsx_add_bool (xml, NULL, value_get_as_int (val));
+					} else {
 						GString *str = g_string_new (NULL);
 						value_get_as_gstring (cell->value, str, state->convs);
 						gsf_xml_out_add_cstr (xml, NULL, str->str);
 						g_string_free (str, TRUE);
-					} else
-						xlsx_add_bool (xml, NULL, val->v_bool.val);
+					}
 					gsf_xml_out_end_element (xml); /* </v> */
 				}
 
@@ -1447,6 +1512,147 @@ xlsx_write_merges (XLSXWriteState *state, GsfXMLOut *xml)
 	}
 }
 
+static void
+xlsx_write_cond_rule (XLSXWriteState *state, GsfXMLOut *xml,
+		      GnmStyleCond *cond, GnmParsePos const *pp)
+{
+	int i, n;
+	const char *type = NULL;
+	const char *operator = NULL;
+	GnmExprTop const *alt_texpr;
+	gboolean needs_alt_texpr = FALSE;
+
+	gsf_xml_out_start_element (xml, "cfRule");
+	switch (cond->op) {
+	case GNM_STYLE_COND_BETWEEN:
+		n = 2; operator = "between";
+		break;
+	case GNM_STYLE_COND_NOT_BETWEEN:
+		n = 2; operator = "notBetween";
+		break;
+	case GNM_STYLE_COND_EQUAL:
+		n = 1; operator = "equal";
+		break;
+	case GNM_STYLE_COND_NOT_EQUAL:
+		n = 1; operator = "notEqual";
+		break;
+	case GNM_STYLE_COND_GT:
+		n = 1; operator = "greaterThan";
+		break;
+	case GNM_STYLE_COND_LT:
+		n = 1; operator = "lessThan";
+		break;
+	case GNM_STYLE_COND_GTE:
+		n = 1; operator = "greaterThanOrEqual";
+		break;
+	case GNM_STYLE_COND_LTE:
+		n = 1; operator = "lessThanOrEqual";
+		break;
+	case GNM_STYLE_COND_CONTAINS_BLANKS:
+		needs_alt_texpr = TRUE;
+		n = 1; type = "containsBlanks";
+		break;
+	case GNM_STYLE_COND_NOT_CONTAINS_BLANKS:
+		needs_alt_texpr = TRUE;
+		n = 1; type = "notContainsBlanks";
+		break;
+	case GNM_STYLE_COND_CONTAINS_ERR:
+		needs_alt_texpr = TRUE;
+		n = 1; type = "containsErrors";
+		break;
+	case GNM_STYLE_COND_NOT_CONTAINS_ERR:
+		needs_alt_texpr = TRUE;
+		n = 1; type = "notContainsErrors";
+		break;
+	case GNM_STYLE_COND_CUSTOM:
+		n = 1; type = "expression";
+		break;
+	case GNM_STYLE_COND_CONTAINS_STR:
+		needs_alt_texpr = TRUE;
+		n = 1; type = "containsText";
+		break;
+	case GNM_STYLE_COND_NOT_CONTAINS_STR:
+		needs_alt_texpr = TRUE;
+		n = 1; type = "notContainsText";
+		break;
+	case GNM_STYLE_COND_BEGINS_WITH_STR:
+		needs_alt_texpr = TRUE;
+		n = 1; type = "beginsWith";
+		break;
+	case GNM_STYLE_COND_ENDS_WITH_STR:
+		needs_alt_texpr = TRUE;
+		n = 1; type = "endsWith";
+		break;
+
+	case GNM_STYLE_COND_NOT_BEGINS_WITH_STR:
+	case GNM_STYLE_COND_NOT_ENDS_WITH_STR:
+		needs_alt_texpr = TRUE;
+		n = 1; type = "expression";
+		break;
+
+	default:
+		g_assert_not_reached ();
+	}
+
+	alt_texpr = needs_alt_texpr
+		? gnm_style_cond_get_alternate_expr (cond)
+		: NULL;
+
+	gsf_xml_out_add_cstr_unchecked (xml, "type", type ? type : "cellIs");
+	gsf_xml_out_add_int (xml, "dxfId",
+			     xlsx_get_cond_style_id (state, cond->overlay));
+	gsf_xml_out_add_int (xml, "priority", 1) ;
+	gsf_xml_out_add_int (xml, "stopIfTrue", 1) ;
+	if (operator)
+		gsf_xml_out_add_cstr_unchecked (xml, "operator", operator);
+	for (i = 0; i < n; i++) {
+		GnmExprTop const *texpr = alt_texpr
+			? alt_texpr
+			: gnm_style_cond_get_expr (cond, i);
+		char *str = gnm_expr_top_as_string (texpr, pp, state->convs);
+		gsf_xml_out_simple_element (xml, "formula", str);
+		g_free (str);
+	}
+	if (alt_texpr)
+		gnm_expr_top_unref (alt_texpr);
+	gsf_xml_out_end_element (xml); /* </cfRule> */
+}
+
+
+static void
+xlsx_write_conditional_formatting (XLSXWriteState *state, GsfXMLOut *xml)
+{
+	GnmStyleList *cond_styles =
+		sheet_style_collect_conditions (state->sheet, NULL);
+	GnmStyleList *l;
+	if (!cond_styles)
+		return;
+
+	for (l = cond_styles; l; l = l->next) {
+		GnmStyleRegion const *sr = l->data;
+		GnmStyleConditions *sc = gnm_style_get_conditions (sr->style);
+		GPtrArray const *conds = gnm_style_conditions_details (sc);
+		unsigned ui;
+		GnmParsePos pp;
+
+		if (!conds)
+			continue;
+
+		parse_pos_init (&pp, NULL, state->sheet,
+				sr->range.start.col, sr->range.start.row);
+		gsf_xml_out_start_element (xml, "conditionalFormatting");
+		xlsx_add_range (xml, "sqref", &sr->range);
+		for (ui = 0; ui < conds->len; ui++) {
+			GnmStyleCond *cond = g_ptr_array_index (conds, ui);
+			xlsx_write_cond_rule (state, xml, cond, &pp);
+		}
+		gsf_xml_out_end_element (xml); /* </conditionalFormatting> */
+	}
+
+	style_list_free (cond_styles);
+}
+
+
 static void
 xlsx_write_validation_expr (XLSXClosure *info, GnmCellPos const *pos,
 			    char const *elem, GnmExprTop const *texpr)
@@ -2449,6 +2655,7 @@ xlsx_write_sheet (XLSXWriteState *state, GsfOutfile *dir, GsfOutfile *wb_part, u
 	xlsx_write_merges (state, xml);
 /*   element phoneticPr { CT_PhoneticPr }?,     */
 /*   element conditionalFormatting { CT_ConditionalFormatting }*,     */
+	xlsx_write_conditional_formatting (state, xml);
 /*   element dataValidations { CT_DataValidations }?,     */
 	xlsx_write_validations (state, xml, &extent);
 /*   element hyperlinks { CT_Hyperlinks }?,     */
@@ -2580,6 +2787,8 @@ xlsx_write_workbook (XLSXWriteState *state, GsfOutfile *root_part)
 	state->shared_string_array = g_ptr_array_new ();
 	state->styles_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
 	state->styles_array = g_ptr_array_new ();
+	state->dxfs_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	state->dxfs_array = g_ptr_array_new ();
 
 	xlsx_get_style_id (state, style);
 	gnm_style_unref (style);
@@ -2675,6 +2884,8 @@ xlsx_write_workbook (XLSXWriteState *state, GsfOutfile *root_part)
 	g_ptr_array_free (state->shared_string_array, TRUE);
 	g_hash_table_destroy (state->styles_hash);
 	g_ptr_array_free (state->styles_array, TRUE);
+	g_hash_table_destroy (state->dxfs_hash);
+	g_ptr_array_free (state->dxfs_array, TRUE);
 
 	if (NULL != state->chart.dir)
 		gsf_output_close (GSF_OUTPUT (state->chart.dir));
